@@ -7,9 +7,8 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file. See the AUTHORS file for names of contributors.
 
-#include <stdio.h>
-
 #include <algorithm>
+#include <cstdio>
 #include <string>
 
 #include "db/db_test_util.h"
