@@ -213,9 +213,9 @@ public class DBOptions extends RocksObject
   }
 
   @Override
-  public DBOptions setLogger(final Logger logger) {
+  public DBOptions setLogger(final LoggerInterface logger) {
     assert(isOwningHandle());
-    setLogger(nativeHandle_, logger.nativeHandle_);
+    setLogger(nativeHandle_, logger.getNativeHandle(), logger.getLoggerType().getValue());
     return this;
   }
 
@@ -1275,8 +1275,7 @@ public class DBOptions extends RocksObject
       long rateLimiterHandle);
   private native void setSstFileManager(final long handle,
       final long sstFileManagerHandle);
-  private native void setLogger(long handle,
-      long loggerHandle);
+  private native void setLogger(final long handle, final long loggerHandle, final byte loggerType);
   private native void setInfoLogLevel(long handle, byte logLevel);
   private native byte infoLogLevel(long handle);
   private native void setMaxOpenFiles(long handle, int maxOpenFiles);
