@@ -573,7 +573,6 @@ public class ReadOptions extends RocksObject {
    * @see #iterStartTs()
    * @return Reference to timestamp or null if there is no timestamp defined.
    */
-  @SuppressWarnings("PMD.ConfusingTernary")
   public Slice timestamp() {
     assert (isOwningHandle());
     final long timestampSliceHandle = timestamp(nativeHandle_);
@@ -623,7 +622,6 @@ public class ReadOptions extends RocksObject {
    * @return Reference to lower bound timestamp or null if there is no lower bound timestamp
    *     defined.
    */
-  @SuppressWarnings("PMD.ConfusingTernary")
   public Slice iterStartTs() {
     assert (isOwningHandle());
     final long iterStartTsHandle = iterStartTs(nativeHandle_);
@@ -752,6 +750,35 @@ public class ReadOptions extends RocksObject {
     return this;
   }
 
+  /**
+   * If async_io is enabled, RocksDB will prefetch some of data asynchronously.
+   * RocksDB apply it if reads are sequential and its internal automatic
+   * prefetching.
+   * <p>
+   * Default: false
+   * @return true if async_io is enabled.
+   */
+  @Experimental("Caution: this option is experimental")
+  public boolean asyncIo() {
+    assert (isOwningHandle());
+    return asyncIo(nativeHandle_);
+  }
+
+  /**
+   * If async_io is enabled, RocksDB will prefetch some of data asynchronously.
+   * RocksDB apply it if reads are sequential and its internal automatic
+   * prefetching.
+   * <p>
+   * @param asyncIo async_io enabled or not.
+   * @return the reference to the current ReadOptions.
+   */
+  @Experimental("Caution: this option is experimental")
+  public ReadOptions setAsyncIo(final boolean asyncIo) {
+    assert (isOwningHandle());
+    setAsyncIo(nativeHandle_, asyncIo);
+    return this;
+  }
+
   // instance variables
   // NOTE: If you add new member variables, please update the copy constructor above!
   //
@@ -819,4 +846,6 @@ public class ReadOptions extends RocksObject {
   private native void setIoTimeout(final long handle, final long ioTimeout);
   private native long valueSizeSoftLimit(final long handle);
   private native void setValueSizeSoftLimit(final long handle, final long softLimit);
+  private native boolean asyncIo(final long handle);
+  private native void setAsyncIo(final long handle, final boolean asyncIO);
 }
