@@ -5,9 +5,8 @@
 
 #include "table/cuckoo/cuckoo_table_builder.h"
 
-#include <assert.h>
-
 #include <algorithm>
+#include <cassert>
 #include <limits>
 #include <string>
 #include <vector>
@@ -319,15 +318,16 @@ Status CuckooTableBuilder::Finish() {
   unused_bucket.resize(static_cast<size_t>(bucket_size), 'a');
   // Write the table.
   uint32_t num_added = 0;
+  const IOOptions opts;
   for (auto& bucket : buckets) {
     if (bucket.vector_idx == kMaxVectorIdx) {
-      io_status_ = file_->Append(Slice(unused_bucket));
+      io_status_ = file_->Append(opts, Slice(unused_bucket));
     } else {
       ++num_added;
-      io_status_ = file_->Append(GetKey(bucket.vector_idx));
+      io_status_ = file_->Append(opts, GetKey(bucket.vector_idx));
       if (io_status_.ok()) {
         if (value_size_ > 0) {
-          io_status_ = file_->Append(GetValue(bucket.vector_idx));
+          io_status_ = file_->Append(opts, GetValue(bucket.vector_idx));
         }
       }
     }
@@ -383,7 +383,7 @@ Status CuckooTableBuilder::Finish() {
   BlockHandle property_block_handle;
   property_block_handle.set_offset(offset);
   property_block_handle.set_size(property_block.size());
-  io_status_ = file_->Append(property_block);
+  io_status_ = file_->Append(opts, property_block);
   offset += property_block.size();
   if (!io_status_.ok()) {
     status_ = io_status_;
@@ -396,7 +396,7 @@ Status CuckooTableBuilder::Finish() {
   BlockHandle meta_index_block_handle;
   meta_index_block_handle.set_offset(offset);
   meta_index_block_handle.set_size(meta_index_block.size());
-  io_status_ = file_->Append(meta_index_block);
+  io_status_ = file_->Append(opts, meta_index_block);
   if (!io_status_.ok()) {
     status_ = io_status_;
     return status_;
@@ -409,7 +409,7 @@ Status CuckooTableBuilder::Finish() {
     status_ = s;
     return status_;
   }
-  io_status_ = file_->Append(footer.GetSlice());
+  io_status_ = file_->Append(opts, footer.GetSlice());
   status_ = io_status_;
   return status_;
 }
@@ -481,7 +481,7 @@ bool CuckooTableBuilder::MakeSpaceForKey(
     uint64_t bid = hash_vals[hash_cnt];
     (*buckets)[static_cast<size_t>(bid)].make_space_for_key_call_id =
         make_space_for_key_call_id;
-    tree.push_back(CuckooNode(bid, 0, 0));
+    tree.emplace_back(bid, 0, 0);
   }
   bool null_found = false;
   uint32_t curr_pos = 0;
@@ -507,7 +507,7 @@ bool CuckooTableBuilder::MakeSpaceForKey(
         }
         (*buckets)[static_cast<size_t>(child_bucket_id)]
             .make_space_for_key_call_id = make_space_for_key_call_id;
-        tree.push_back(CuckooNode(child_bucket_id, curr_depth + 1, curr_pos));
+        tree.emplace_back(child_bucket_id, curr_depth + 1, curr_pos);
         if ((*buckets)[static_cast<size_t>(child_bucket_id)].vector_idx ==
             kMaxVectorIdx) {
           null_found = true;
