@@ -98,11 +98,13 @@ void TableReaderBenchmark(Options& opts, EnvOptions& env_options,
     IntTblPropCollectorFactories int_tbl_prop_collector_factories;
 
     int unknown_level = -1;
+    const WriteOptions write_options;
     tb = opts.table_factory->NewTableBuilder(
-        TableBuilderOptions(
-            ioptions, moptions, ikc, &int_tbl_prop_collector_factories,
-            CompressionType::kNoCompression, CompressionOptions(),
-            0 /* column_family_id */, kDefaultColumnFamilyName, unknown_level),
+        TableBuilderOptions(ioptions, moptions, read_options, write_options,
+                            ikc, &int_tbl_prop_collector_factories,
+                            CompressionType::kNoCompression,
+                            CompressionOptions(), 0 /* column_family_id */,
+                            kDefaultColumnFamilyName, unknown_level),
         file_writer.get());
   } else {
     s = DB::Open(opts, dbname, &db);
@@ -122,7 +124,7 @@ void TableReaderBenchmark(Options& opts, EnvOptions& env_options,
   }
   if (!through_db) {
     tb->Finish();
-    file_writer->Close();
+    file_writer->Close(IOOptions());
   } else {
     db->Flush(FlushOptions());
   }
