@@ -23,6 +23,7 @@
 #include "logging/logging.h"
 #include "monitoring/instrumented_mutex.h"
 #include "monitoring/statistics_impl.h"
+#include "monitoring/thread_status_util.h"
 #include "rocksdb/convenience.h"
 #include "rocksdb/env.h"
 #include "rocksdb/iterator.h"
@@ -106,6 +107,15 @@ BlobDBImpl::~BlobDBImpl() {
 }
 
 Status BlobDBImpl::Close() {
+  ThreadStatus::OperationType cur_op_type =
+      ThreadStatusUtil::GetThreadOperation();
+  ThreadStatusUtil::SetThreadOperation(ThreadStatus::OperationType::OP_UNKNOWN);
+  Status s = CloseImpl();
+  ThreadStatusUtil::SetThreadOperation(cur_op_type);
+  return s;
+}
+
+Status BlobDBImpl::CloseImpl() {
   if (closed_) {
     return Status::OK();
   }
@@ -123,7 +133,8 @@ Status BlobDBImpl::Close() {
     return s;
   }
 
-  s = SyncBlobFiles();
+  // TODO: plumb Env::IOActivity, Env::IOPriority
+  s = SyncBlobFiles(WriteOptions());
   return s;
 }
 
@@ -277,7 +288,7 @@ Status BlobDBImpl::Open(std::vector<ColumnFamilyHandle*>* handles) {
     return s;
   }
 
-  UpdateLiveSSTSize();
+  UpdateLiveSSTSize(WriteOptions(Env::IOActivity::kDBOpen));
 
   // Start background jobs.
   if (!bdb_options_.disable_background_tasks) {
@@ -743,7 +754,9 @@ Status BlobDBImpl::CreateWriterLocked(const std::shared_ptr<BlobFile>& bfile) {
   }
 
   std::unique_ptr<WritableFileWriter> fwriter;
-  fwriter.reset(new WritableFileWriter(std::move(wfile), fpath, file_options_));
+  fwriter.reset(new WritableFileWriter(
+      std::move(wfile), fpath, file_options_, clock_, nullptr /* io_tracer */,
+      statistics_, Histograms::BLOB_DB_BLOB_FILE_WRITE_MICROS));
 
   uint64_t boffset = bfile->GetFileSize();
   if (debug_level_ >= 2 && boffset) {
@@ -824,8 +837,9 @@ Status BlobDBImpl::CheckOrCreateWriterLocked(
 }
 
 Status BlobDBImpl::CreateBlobFileAndWriter(
-    bool has_ttl, const ExpirationRange& expiration_range,
-    const std::string& reason, std::shared_ptr<BlobFile>* blob_file,
+    const WriteOptions& write_options, bool has_ttl,
+    const ExpirationRange& expiration_range, const std::string& reason,
+    std::shared_ptr<BlobFile>* blob_file,
     std::shared_ptr<BlobLogWriter>* writer) {
   TEST_SYNC_POINT("BlobDBImpl::CreateBlobFileAndWriter");
   assert(has_ttl == (expiration_range.first || expiration_range.second));
@@ -846,7 +860,7 @@ Status BlobDBImpl::CreateBlobFileAndWriter(
 
   assert(*writer);
 
-  s = (*writer)->WriteHeader((*blob_file)->header_);
+  s = (*writer)->WriteHeader(write_options, (*blob_file)->header_);
   if (!s.ok()) {
     ROCKS_LOG_ERROR(db_options_.info_log,
                     "Failed to write header to new blob file: %s"
@@ -861,7 +875,8 @@ Status BlobDBImpl::CreateBlobFileAndWriter(
   return s;
 }
 
-Status BlobDBImpl::SelectBlobFile(std::shared_ptr<BlobFile>* blob_file) {
+Status BlobDBImpl::SelectBlobFile(const WriteOptions& write_options,
+                                  std::shared_ptr<BlobFile>* blob_file) {
   assert(blob_file);
 
   {
@@ -885,6 +900,7 @@ Status BlobDBImpl::SelectBlobFile(std::shared_ptr<BlobFile>* blob_file) {
 
   std::shared_ptr<BlobLogWriter> writer;
   const Status s = CreateBlobFileAndWriter(
+      write_options,
       /* has_ttl */ false, ExpirationRange(),
       /* reason */ "SelectBlobFile", blob_file, &writer);
   if (!s.ok()) {
@@ -897,7 +913,8 @@ Status BlobDBImpl::SelectBlobFile(std::shared_ptr<BlobFile>* blob_file) {
   return s;
 }
 
-Status BlobDBImpl::SelectBlobFileTTL(uint64_t expiration,
+Status BlobDBImpl::SelectBlobFileTTL(const WriteOptions& write_options,
+                                     uint64_t expiration,
                                      std::shared_ptr<BlobFile>* blob_file) {
   assert(blob_file);
   assert(expiration != kNoExpiration);
@@ -930,9 +947,9 @@ Status BlobDBImpl::SelectBlobFileTTL(uint64_t expiration,
   oss << "SelectBlobFileTTL range: [" << exp_low << ',' << exp_high << ')';
 
   std::shared_ptr<BlobLogWriter> writer;
-  const Status s =
-      CreateBlobFileAndWriter(/* has_ttl */ true, expiration_range,
-                              /* reason */ oss.str(), blob_file, &writer);
+  const Status s = CreateBlobFileAndWriter(
+      write_options, /* has_ttl */ true, expiration_range,
+      /* reason */ oss.str(), blob_file, &writer);
   if (!s.ok()) {
     return s;
   }
@@ -1055,7 +1072,7 @@ Status BlobDBImpl::PutUntil(const WriteOptions& options, const Slice& key,
   return s;
 }
 
-Status BlobDBImpl::PutBlobValue(const WriteOptions& /*options*/,
+Status BlobDBImpl::PutBlobValue(const WriteOptions& write_options,
                                 const Slice& key, const Slice& value,
                                 uint64_t expiration, WriteBatch* batch) {
   write_mutex_.AssertHeld();
@@ -1087,30 +1104,30 @@ Status BlobDBImpl::PutBlobValue(const WriteOptions& /*options*/,
     // Check DB size limit before selecting blob file to
     // Since CheckSizeAndEvictBlobFiles() can close blob files, it needs to be
     // done before calling SelectBlobFile().
-    s = CheckSizeAndEvictBlobFiles(headerbuf.size() + key.size() +
-                                   value_compressed.size());
+    s = CheckSizeAndEvictBlobFiles(
+        write_options, headerbuf.size() + key.size() + value_compressed.size());
     if (!s.ok()) {
       return s;
     }
 
     std::shared_ptr<BlobFile> blob_file;
     if (expiration != kNoExpiration) {
-      s = SelectBlobFileTTL(expiration, &blob_file);
+      s = SelectBlobFileTTL(write_options, expiration, &blob_file);
     } else {
-      s = SelectBlobFile(&blob_file);
+      s = SelectBlobFile(write_options, &blob_file);
     }
     if (s.ok()) {
       assert(blob_file != nullptr);
       assert(blob_file->GetCompressionType() == bdb_options_.compression);
-      s = AppendBlob(blob_file, headerbuf, key, value_compressed, expiration,
-                     &index_entry);
+      s = AppendBlob(write_options, blob_file, headerbuf, key, value_compressed,
+                     expiration, &index_entry);
     }
     if (s.ok()) {
       if (expiration != kNoExpiration) {
         WriteLock file_lock(&blob_file->mutex_);
         blob_file->ExtendExpirationRange(expiration);
       }
-      s = CloseBlobFileIfNeeded(blob_file);
+      s = CloseBlobFileIfNeeded(write_options, blob_file);
     }
     if (s.ok()) {
       s = WriteBatchInternal::PutBlobIndex(batch, column_family_id, key,
@@ -1249,7 +1266,7 @@ void BlobDBImpl::GetCompactionContext(BlobCompactionContext* context,
   }
 }
 
-void BlobDBImpl::UpdateLiveSSTSize() {
+void BlobDBImpl::UpdateLiveSSTSize(const WriteOptions& write_options) {
   uint64_t live_sst_size = 0;
   bool ok = GetIntProperty(DB::Properties::kLiveSstFilesSize, &live_sst_size);
   if (ok) {
@@ -1265,7 +1282,7 @@ void BlobDBImpl::UpdateLiveSSTSize() {
   {
     // Trigger FIFO eviction if needed.
     MutexLock l(&write_mutex_);
-    Status s = CheckSizeAndEvictBlobFiles(0, true /*force*/);
+    Status s = CheckSizeAndEvictBlobFiles(write_options, 0, true /*force*/);
     if (s.IsNoSpace()) {
       ROCKS_LOG_WARN(db_options_.info_log,
                      "DB grow out-of-space after SST size updated. Current live"
@@ -1276,7 +1293,8 @@ void BlobDBImpl::UpdateLiveSSTSize() {
   }
 }
 
-Status BlobDBImpl::CheckSizeAndEvictBlobFiles(uint64_t blob_size,
+Status BlobDBImpl::CheckSizeAndEvictBlobFiles(const WriteOptions& write_options,
+                                              uint64_t blob_size,
                                               bool force_evict) {
   write_mutex_.AssertHeld();
 
@@ -1316,7 +1334,7 @@ Status BlobDBImpl::CheckSizeAndEvictBlobFiles(uint64_t blob_size,
     }
     // FIFO eviction can evict open blob files.
     if (!blob_file->Immutable()) {
-      Status s = CloseBlobFile(blob_file);
+      Status s = CloseBlobFile(write_options, blob_file);
       if (!s.ok()) {
         return s;
       }
@@ -1347,7 +1365,8 @@ Status BlobDBImpl::CheckSizeAndEvictBlobFiles(uint64_t blob_size,
   return Status::OK();
 }
 
-Status BlobDBImpl::AppendBlob(const std::shared_ptr<BlobFile>& bfile,
+Status BlobDBImpl::AppendBlob(const WriteOptions& write_options,
+                              const std::shared_ptr<BlobFile>& bfile,
                               const std::string& headerbuf, const Slice& key,
                               const Slice& value, uint64_t expiration,
                               std::string* index_entry) {
@@ -1363,8 +1382,8 @@ Status BlobDBImpl::AppendBlob(const std::shared_ptr<BlobFile>& bfile,
     }
 
     // write the blob to the blob log.
-    s = writer->EmitPhysicalRecord(headerbuf, key, value, &key_offset,
-                                   &blob_offset);
+    s = writer->EmitPhysicalRecord(write_options, headerbuf, key, value,
+                                   &key_offset, &blob_offset);
   }
 
   if (!s.ok()) {
@@ -1767,7 +1786,8 @@ std::pair<bool, int64_t> BlobDBImpl::SanityCheck(bool aborted) {
   return std::make_pair(true, -1);
 }
 
-Status BlobDBImpl::CloseBlobFile(std::shared_ptr<BlobFile> bfile) {
+Status BlobDBImpl::CloseBlobFile(const WriteOptions& write_options,
+                                 std::shared_ptr<BlobFile> bfile) {
   TEST_SYNC_POINT("BlobDBImpl::CloseBlobFile");
   assert(bfile);
   assert(!bfile->Immutable());
@@ -1783,7 +1803,7 @@ Status BlobDBImpl::CloseBlobFile(std::shared_ptr<BlobFile> bfile) {
 
   const SequenceNumber sequence = GetLatestSequenceNumber();
 
-  const Status s = bfile->WriteFooterAndCloseLocked(sequence);
+  const Status s = bfile->WriteFooterAndCloseLocked(write_options, sequence);
 
   if (s.ok()) {
     total_blob_size_ += BlobLogFooter::kSize;
@@ -1815,7 +1835,8 @@ Status BlobDBImpl::CloseBlobFile(std::shared_ptr<BlobFile> bfile) {
   return s;
 }
 
-Status BlobDBImpl::CloseBlobFileIfNeeded(std::shared_ptr<BlobFile>& bfile) {
+Status BlobDBImpl::CloseBlobFileIfNeeded(const WriteOptions& write_options,
+                                         std::shared_ptr<BlobFile>& bfile) {
   write_mutex_.AssertHeld();
 
   // atomic read
@@ -1831,7 +1852,7 @@ Status BlobDBImpl::CloseBlobFileIfNeeded(std::shared_ptr<BlobFile>& bfile) {
     return Status::OK();
   }
 
-  return CloseBlobFile(bfile);
+  return CloseBlobFile(write_options, bfile);
 }
 
 void BlobDBImpl::ObsoleteBlobFile(std::shared_ptr<BlobFile> blob_file,
@@ -1921,7 +1942,8 @@ std::pair<bool, int64_t> BlobDBImpl::EvictExpiredFiles(bool aborted) {
       }
 
       if (!blob_file->Immutable()) {
-        CloseBlobFile(blob_file).PermitUncheckedError();
+        // TODO: plumb Env::IOActivity, Env::IOPriority
+        CloseBlobFile(WriteOptions(), blob_file).PermitUncheckedError();
       }
 
       assert(blob_file->Immutable());
@@ -1933,7 +1955,7 @@ std::pair<bool, int64_t> BlobDBImpl::EvictExpiredFiles(bool aborted) {
   return std::make_pair(true, -1);
 }
 
-Status BlobDBImpl::SyncBlobFiles() {
+Status BlobDBImpl::SyncBlobFiles(const WriteOptions& write_options) {
   MutexLock l(&write_mutex_);
 
   std::vector<std::shared_ptr<BlobFile>> process_files;
@@ -1949,7 +1971,7 @@ Status BlobDBImpl::SyncBlobFiles() {
 
   Status s;
   for (auto& blob_file : process_files) {
-    s = blob_file->Fsync();
+    s = blob_file->Fsync(write_options);
     if (!s.ok()) {
       ROCKS_LOG_ERROR(db_options_.info_log,
                       "Failed to sync blob file %" PRIu64 ", status: %s",
@@ -2196,7 +2218,7 @@ Status BlobDBImpl::TEST_CloseBlobFile(std::shared_ptr<BlobFile>& bfile) {
   WriteLock lock(&mutex_);
   WriteLock file_lock(&bfile->mutex_);
 
-  return CloseBlobFile(bfile);
+  return CloseBlobFile(WriteOptions(), bfile);
 }
 
 void BlobDBImpl::TEST_ObsoleteBlobFile(std::shared_ptr<BlobFile>& blob_file,
