@@ -40,7 +40,7 @@ enum CacheDumpUnitType : unsigned char {
   kBlockTypeMax,
 };
 
-// The metadata of a dump unit. After it is serilized, its size is fixed 16
+// The metadata of a dump unit. After it is serialized, its size is fixed 16
 // bytes.
 struct DumpUnitMeta {
   // sequence number is a monotonically increasing number to indicate the order
@@ -48,7 +48,7 @@ struct DumpUnitMeta {
   uint32_t sequence_num;
   // The Crc32c checksum of its dump unit.
   uint32_t dump_unit_checksum;
-  // The dump unit size after the dump unit is serilized to a string.
+  // The dump unit size after the dump unit is serialized to a string.
   uint64_t dump_unit_size;
 
   void reset() {
@@ -162,11 +162,12 @@ class ToFileCacheDumpWriter : public CacheDumpWriter {
     assert(file_writer_ != nullptr);
     std::string prefix;
     PutFixed32(&prefix, static_cast<uint32_t>(metadata.size()));
-    IOStatus io_s = file_writer_->Append(Slice(prefix));
+    const IOOptions opts;
+    IOStatus io_s = file_writer_->Append(opts, Slice(prefix));
     if (!io_s.ok()) {
       return io_s;
     }
-    io_s = file_writer_->Append(metadata);
+    io_s = file_writer_->Append(opts, metadata);
     return io_s;
   }
 
@@ -175,11 +176,12 @@ class ToFileCacheDumpWriter : public CacheDumpWriter {
     assert(file_writer_ != nullptr);
     std::string prefix;
     PutFixed32(&prefix, static_cast<uint32_t>(data.size()));
-    IOStatus io_s = file_writer_->Append(Slice(prefix));
+    const IOOptions opts;
+    IOStatus io_s = file_writer_->Append(opts, Slice(prefix));
     if (!io_s.ok()) {
       return io_s;
     }
-    io_s = file_writer_->Append(data);
+    io_s = file_writer_->Append(opts, data);
     return io_s;
   }
 
